--	ActiveWeaponIcon - LUA
--
--	@author: @Iyanga
--	File:    ActiveWeaponIcon.lua
--	Version: 1.3
--  Slash command: /awi
--  Supported paramters: style - toggles between the different supported styles


local ActiveWeaponIcon = { }

ActiveWeaponIcon.STYLE_ICON   = 1
ActiveWeaponIcon.STYLE_NUMBER = 2
ActiveWeaponIcon.STYLE_ROTATE = 3

ActiveWeaponIcon.defaultValues =
{
  relativePoint = nil,
  point   = CENTER,
  offsetX = -100,
  offsetY = 0,
  style = ActiveWeaponIcon.STYLE_NUMBER
}

--- Check whether a string starts with a specific token
-- @param text  [string] The string to test
-- @param token [string] The token to find
-- @return true if found, false if not
local function StartsWith(text, token)
  if(type(text) == "string") then
    return string.sub(text, 1, string.len(token)) == token
  end
end


-- Reticle hidden update
function ActiveWeaponIcon.OnReticleHiddenUpdate(eventCode, hidden)
  ActiveWeaponIconControlText:SetHidden(hidden)
end


-- Call this function __after__ the style value was changed
function ActiveWeaponIcon.OnUpdateStyle(style)
  local activeWeaponPair, _ = GetActiveWeaponPairInfo()
  ActiveWeaponIcon.UpdateIcon(activeWeaponPair)
  if style == ActiveWeaponIcon.STYLE_NUMBER then
      ZO_ReticleContainerReticle:SetTextureRotation(0.0, 0.5, 0.5)
      ActiveWeaponIconControlBG:SetHidden(true)  
      ActiveWeaponIconControlIcon:SetHidden(true)        
      ActiveWeaponIconControlText:SetHidden(ZO_ReticleContainerReticle:IsHidden())
      EVENT_MANAGER:RegisterForEvent("ActiveWeaponIcon", EVENT_RETICLE_HIDDEN_UPDATE, ActiveWeaponIcon.OnReticleHiddenUpdate)            
  elseif style == ActiveWeaponIcon.STYLE_ICON then
      ZO_ReticleContainerReticle:SetTextureRotation(0.0, 0.5, 0.5)
      ActiveWeaponIconControlBG:SetHidden(false)      
      ActiveWeaponIconControlIcon:SetHidden(false)
      ActiveWeaponIconControlText:SetHidden(true)
      EVENT_MANAGER:UnregisterForEvent("ActiveWeaponIcon", EVENT_RETICLE_HIDDEN_UPDATE)
  elseif style == ActiveWeaponIcon.STYLE_ROTATE then
      ActiveWeaponIconControlBG:SetHidden(true)      
      ActiveWeaponIconControlIcon:SetHidden(true)
      ActiveWeaponIconControlText:SetHidden(true)
      EVENT_MANAGER:UnregisterForEvent("ActiveWeaponIcon", EVENT_RETICLE_HIDDEN_UPDATE)
  end  
end    


--- Sets the slash command for ActiveWeaponIcon
-- No useful commands supported yet.
SLASH_COMMANDS["/awi"] = function(text)
  if text == "" then
     d("ActiveWeaponIcon: Running.") 
  elseif StartsWith(text, "help") then
    d("ActiveWeaponIcon: No additional slash commands supported right now.")    
  elseif StartsWith(text, "style") then
    if ActiveWeaponIcon.savedValues.style == ActiveWeaponIcon.STYLE_NUMBER then
        ActiveWeaponIcon.savedValues.style = ActiveWeaponIcon.STYLE_ICON        
        d("ActiveWeaponIcon: Icon style enabled.")
    elseif ActiveWeaponIcon.savedValues.style == ActiveWeaponIcon.STYLE_ICON then
      ActiveWeaponIcon.savedValues.style = ActiveWeaponIcon.STYLE_ROTATE    
      d("ActiveWeaponIcon: Rotate style enabled.")      
    elseif ActiveWeaponIcon.savedValues.style == ActiveWeaponIcon.STYLE_ROTATE then
      ActiveWeaponIcon.savedValues.style = ActiveWeaponIcon.STYLE_NUMBER      
      d("ActiveWeaponIcon: Number style enabled.")      
    end
    ActiveWeaponIcon.OnUpdateStyle(ActiveWeaponIcon.savedValues.style)
  elseif StartsWith(text, "who") then
    d("ActiveWeaponIcon: ActiveWeaponIcon Addon by @Iyanga")  
  else
    d("ActiveWeaponIcon: Unsupported command.")
  end
end



function ActiveWeaponIcon.UpdateIcon(activeWeaponPair)
  if ActiveWeaponIcon.savedValues.style == ActiveWeaponIcon.STYLE_ICON then  
    local displaySlot, color  
    
    if activeWeaponPair == ACTIVE_WEAPON_PAIR_MAIN then
      displaySlot = EQUIP_SLOT_MAIN_HAND
      color       = ZO_ColorDef:New( 1.0, 0.0, 0.0, 1.0 )      
    elseif activeWeaponPair == ACTIVE_WEAPON_PAIR_BACKUP then
      displaySlot = EQUIP_SLOT_BACKUP_MAIN
      color       = ZO_ColorDef:New( 0.0, 1.0, 0.0, 1.0 )  
    else
      return 
    end
    
    local icon, slotHasItem, sellPrice, isHeldSlot, isHeldNow, locked = GetEquippedItemInfo(displaySlot)
      
    ActiveWeaponIconControlIcon:SetTexture(icon)  
    ActiveWeaponIconControlIcon:SetColor(color:UnpackRGBA())    
  elseif ActiveWeaponIcon.savedValues.style == ActiveWeaponIcon.STYLE_NUMBER then
    ActiveWeaponIconControlText:SetText(tostring(activeWeaponPair))
  elseif ActiveWeaponIcon.savedValues.style == ActiveWeaponIcon.STYLE_ROTATE then
    local rotateValue
    if activeWeaponPair == ACTIVE_WEAPON_PAIR_MAIN then
      rotateValue = 0.0
    else
      rotateValue = 0.785398163
    end    
    ZO_ReticleContainerReticle:SetTextureRotation(rotateValue, 0.5, 0.5)  
  else
    d("ActiveWeaponIcon: Unexpected code path.")    
  end    
    
end


--- Control Event OnMoveStop
function ActiveWeaponIcon.OnMoveStop()
  isValidAnchor, point, relativeTo, relativePoint, offsetX, offsetY = ActiveWeaponIconControl:GetAnchor(0)
  if isValidAnchor == true then
    ActiveWeaponIcon.savedValues.relativePoint  = relativePoint    
    ActiveWeaponIcon.savedValues.point          = point    
    ActiveWeaponIcon.savedValues.offsetX        = offsetX
    ActiveWeaponIcon.savedValues.offsetY        = offsetY
  else
    d("ActiveWeaponIcon: Unexpected invalid anchor found.")  
  end

end


--- EVENT_ACTIVE_WEAPON_PAIR_CHANGED event handler
function ActiveWeaponIcon.WeaponChanged(eventCode, activeWeaponPair, locked)
  ActiveWeaponIcon.UpdateIcon(activeWeaponPair)
end


---	EVENT_ADD_ON_LOADED event handler
function ActiveWeaponIcon.Initialize(self, addOnName)
	
  if(addOnName ~= "ActiveWeaponIcon") then return end
  
  ActiveWeaponIcon.savedValues = ZO_SavedVars:New("ActiveWeaponIcon_SavedVariables", 2, nil, ActiveWeaponIcon.defaultValues, nil)  
  
  ActiveWeaponIconControl:ClearAnchors()
  
  ActiveWeaponIconControl:SetAnchor(ActiveWeaponIcon.savedValues.point, GuiRoot, ActiveWeaponIcon.savedValues.relativePoint, ActiveWeaponIcon.savedValues.offsetX, ActiveWeaponIcon.savedValues.offsetY)
  ActiveWeaponIconControl:SetMouseEnabled(true)
  ActiveWeaponIconControl:SetMovable(true)
  
  ActiveWeaponIconControl:SetDrawLevel(0)
  ActiveWeaponIconControl:SetDrawLayer(0)
  ActiveWeaponIconControl:SetDrawTier(0)
  
  ActiveWeaponIconControl:SetHandler("OnMoveStop", ActiveWeaponIcon.OnMoveStop)  
  
  local activeWeaponPair, _ = GetActiveWeaponPairInfo()
  ActiveWeaponIcon.UpdateIcon(activeWeaponPair)
  
  ActiveWeaponIcon.OnUpdateStyle(ActiveWeaponIcon.savedValues.style)
  		 
  EVENT_MANAGER:RegisterForEvent("ActiveWeaponIcon", EVENT_ACTIVE_WEAPON_PAIR_CHANGED, ActiveWeaponIcon.WeaponChanged)
  EVENT_MANAGER:UnregisterForEvent("ActiveWeaponIcon", EVENT_ADD_ON_LOADED)
end

---  Hook initialization onto the EVENT_ADD_ON_LOADED event
EVENT_MANAGER:RegisterForEvent("ActiveWeaponIcon", EVENT_ADD_ON_LOADED, ActiveWeaponIcon.Initialize)
